/************************************************************************/
/* @\F 4 pPbg_                                        */
/*                                                                      */
/* TvF 4 pPbgs܂                                  */
/*        trclkɓ܂                                             */
/*                                                                      */
/*                                                                  */
/*                                                                      */
/*                                                                      */
/* ύXF                                                           */
/*  +------------------ ԍ (000 ` 999)                           */
/*  |    +------------- VKAύXAǉA폜̕                    */
/*  v    v                                                              */
/*  No     N                                               */
/* ----+------+----------+--------------------------------------------- */
/*  000 VK   16/09/06   Packet_Rev1_03 x[XɐVK쐬               */
/*                        4pPbgΉAAhX1bitǉ           */
/*                                                                      */
/************************************************************************/

module packet_gen(
	trclk,
	trreset,
	dataout,
	empty,
	rd,
	clk,
	xsync,
	d
	);

/***************************************************************************/
/* DECLARE THE PORT DIRECTION                                              */
/***************************************************************************/

 input			trclk;
 input			trreset;
 input	[36:0]	dataout;
 input			empty;
 output			rd;
 output			clk;
 output			xsync;
 output	[1:0]	d;


/***************************************************************************/
/* DECLARE THE NODE(WIRE/REG)                                              */
/***************************************************************************/
 reg			fifo_rd;
 reg	[35:0]	fifo_dat;
 reg			lost;
 reg	[1:0]	fifo_st;

 reg			fifo_dat_valid;

 reg	[4:0]	o_cnt;
 reg	[35:0]	rd_dat;
 reg	[1:0]	adr_keep;
 reg			xvalid;
 reg	[1:0]	pkt_d;
 reg			pkt_end;
 reg	[1:0]	pkt_st;

 wire			packet_type;

/***************************************************************************/
/* patameter                                                               */
/***************************************************************************/
parameter
	WAIT_EMPTY_NEG	= 2'b00,
	WAIT_RD_DAT		= 2'b01,
	GET_FIFO_DAT	= 2'b10,
	WAIT_PKT_END	= 2'b11;

parameter
	WAIT_FIFO_DAT	= 2'b00,
	ADR_OUT0		= 2'b01,
	ADR_OUT1		= 2'b10,
	DAT_OUT			= 2'b11;

parameter
	bus_size	= 2'b10;

parameter
	packet_size	= 5'b10000;


/***************************************************************************/
/* output                                                                  */
/***************************************************************************/
	assign clk		= trclk ;
	assign xsync	= xvalid ;
	assign d		= pkt_d  ;
	assign rd		= fifo_rd;

/***************************************************************************/
/* Packet Generator                                                        */
/***************************************************************************/

	always @(posedge trclk) begin
		if (trreset) begin
			fifo_rd			<= 1'b0;
			fifo_dat[35:0]	<= 0;
			lost			<= 1'b0;
			fifo_st			<= WAIT_EMPTY_NEG;
		end
		else begin
			case (fifo_st)
			WAIT_EMPTY_NEG : begin
				lost			<= 1'b0;
				
				if ( empty == 1'b0 ) begin
					fifo_rd		<= 1'b1;
					fifo_st		<= WAIT_RD_DAT;
				end
				else begin
					fifo_rd		<= 1'b0;
					fifo_st		<= WAIT_EMPTY_NEG;
				end
			end
			WAIT_RD_DAT : begin
				fifo_rd			<= 1'b0;
				lost			<= 1'b0;
				fifo_st			<= GET_FIFO_DAT;
			end
			GET_FIFO_DAT : begin
				fifo_rd			<= 1'b0;
				fifo_dat[35:0]	<= dataout[35:0];
				lost			<= dataout[36];
				fifo_st			<= WAIT_PKT_END;
			end
			WAIT_PKT_END : begin
				if ( pkt_end ) begin
					fifo_rd			<= 1'b0;
					fifo_dat[35:0]	<= fifo_dat[35:0];
					lost			<= 1'b0;
					fifo_st			<= WAIT_EMPTY_NEG;
				end
				else begin
					fifo_rd			<= 1'b0;
					fifo_dat[35:0]	<= fifo_dat[35:0];
					lost			<= lost;
					fifo_st			<= WAIT_PKT_END;
				end
			end
			default : begin
				fifo_rd			<= 1'b0;
				lost			<= 1'b0;
				fifo_st			<= WAIT_EMPTY_NEG;
			end
			endcase
		end
	end

	always @(posedge trclk) begin
		if (trreset) begin
			fifo_dat_valid	<= 1'b0;
		end
		else begin
			fifo_dat_valid	<= ( fifo_st == GET_FIFO_DAT );
		end
	end

	assign	packet_type	= ( {fifo_dat[0],fifo_dat[3:2]} == 3'b000 ) ? 1'b0 :
						  ( {fifo_dat[0],fifo_dat[3:2]} == 3'b001 ) ? 1'b0 : 1'b1;

	always @(posedge trclk) begin
		if (trreset) begin
			o_cnt			<= 0;
			rd_dat[35:0]	<= 0;
			adr_keep[1:0]	<= 0;
			xvalid			<= 1'b1;
			pkt_d[1:0]		<= 0;
			pkt_end			<= 1'b0;
			pkt_st			<= WAIT_FIFO_DAT;
		end
		else begin
			case (pkt_st)
			WAIT_FIFO_DAT : begin
				if ( fifo_dat_valid ) begin
					o_cnt			<= packet_size;
					rd_dat[35:0]	<= fifo_dat[35:0];				//	{ lost, td[31:0], ta[4:2], ta[5] }
					adr_keep[1:0]	<= {fifo_dat[0],fifo_dat[3]};	//	ta[5:4]
					xvalid			<= 1'b1;
					pkt_d[1:0]		<= { lost, packet_type };
					pkt_end			<= 1'b0;
					pkt_st			<= ADR_OUT0;
				end
				else begin
					o_cnt			<= 0;
					rd_dat[35:0]	<= rd_dat[35:0];
					adr_keep[1:0]	<= adr_keep[1:0];
					xvalid			<= 1'b1;
					pkt_d[1:0]		<= 0;
					pkt_end			<= 1'b0;
					pkt_st			<= WAIT_FIFO_DAT;
				end
			end
			ADR_OUT0 : begin
				rd_dat[35:0]	<= { 4'b0000, rd_dat[35:4] };		//	{ 4'b0000, td[31:0] }
				adr_keep[1:0]	<= adr_keep[1:0];
				xvalid			<= 1'b0;
				pkt_d[1:0]		<= rd_dat[2:1];						//	{ ta[3:2] }
				pkt_end			<= 1'b0;
				
				if (o_cnt == 5'b0000 ) begin
					o_cnt		<= 0;
					pkt_st		<= WAIT_FIFO_DAT;
				end
				else begin
					o_cnt		<= o_cnt - 1;
					pkt_st		<= ( packet_type ) ? ADR_OUT1 : DAT_OUT;
				end
			end
			ADR_OUT1 : begin
				rd_dat[35:0]	<= rd_dat[35:0];
				adr_keep[1:0]	<= adr_keep[1:0];
				xvalid			<= 1'b0;
				pkt_d[1:0]		<= adr_keep[1:0];
				pkt_end			<= 1'b0;
				
				if (o_cnt == 5'b0000 ) begin
					o_cnt		<= 0;
					pkt_st		<= WAIT_FIFO_DAT;
				end
				else begin
					o_cnt		<= o_cnt;
					pkt_st		<= DAT_OUT;
				end
			end
			DAT_OUT : begin
				rd_dat[35:0]	<= { 2'b00, rd_dat[35:2] };
				adr_keep[1:0]	<= adr_keep[1:0];
				xvalid			<= 1'b0;
				pkt_d[1:0]		<= rd_dat[1:0];
				
				if (o_cnt == 5'b0000 ) begin
					o_cnt		<= 0;
					pkt_end		<= 1'b0;
					pkt_st		<= WAIT_FIFO_DAT;
				end
				else begin
					o_cnt		<= o_cnt - 1;
					pkt_st		<= DAT_OUT;
					
					if (o_cnt == 5'b0100 ) begin
						pkt_end	<= 1'b1;
					end
					else begin
						pkt_end	<= 1'b0;
					end
				end
			end
			default : begin
				o_cnt			<= 0;
				rd_dat[35:0]	<= 0;
				adr_keep[1:0]	<= 0;
				xvalid			<= 1'b1;
				pkt_d[1:0]		<= 0;
				pkt_end			<= 1'b0;
				pkt_st			<= WAIT_FIFO_DAT;
			end
			endcase
		end
	end


endmodule
