/************************************************************************/
/* @\F pPbg_TOPW[                                 */
/*                                                                      */
/* TvF AXI Litef[^oX:32bit4pPbg֕ϊ܂           */
/*                                                                      */
/*                                                                  */
/*                                                                      */
/*                                                                      */
/* ύXF                                                           */
/*  +------------------ ԍ (000 ` 999)                           */
/*  |    +------------- VKAύXAǉA폜̕                    */
/*  v    v                                                              */
/*  No     N                                               */
/* ----+------+----------+--------------------------------------------- */
/*  000 VK   16/11/30                                                 */
/*  001 ύX   17/05/19   Cgf[^荞ݎS_AXI_WSTRBQƂ */
/*                        悤ɕύX                                    */
/*                        AXIoXMXe[g}V̌             */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/

`timescale 1ns/1ps

module smt_packet_axi_lite_slave #
	(
	parameter		  PACKET_IF_ADR_UPPER_20BIT = 20'h80020,	//Packet I/F Address Upper 20bit
	parameter integer C_S_AXI_ADDR_WIDTH		= 32,
	parameter integer C_S_AXI_DATA_WIDTH		= 32
	)
	(
	// System Signals
	input wire			ACLK,
	input wire			ARESETN,

	// Slave Interface Write Address Ports
	input  wire	[C_S_AXI_ADDR_WIDTH-1:0] S_AXI_AWADDR,
	input  wire	[3-1:0]	S_AXI_AWPROT,
	input  wire			S_AXI_AWVALID,
	output reg			S_AXI_AWREADY,

	// Slave Interface Write Data Ports
	input  wire	[C_S_AXI_DATA_WIDTH-1:0] S_AXI_WDATA,
	input  wire	[C_S_AXI_DATA_WIDTH/8-1:0] S_AXI_WSTRB,
	input  wire			S_AXI_WVALID,
	output reg			S_AXI_WREADY,

	// Slave Interface Write Response Ports
	output reg	[2-1:0]	S_AXI_BRESP,
	output reg			S_AXI_BVALID,
	input  wire			S_AXI_BREADY,

	// Slave Interface Read Address Ports
	input  wire	[C_S_AXI_ADDR_WIDTH-1:0] S_AXI_ARADDR,
	input  wire	[3-1:0]	S_AXI_ARPROT,
	input  wire			S_AXI_ARVALID,
	output reg			S_AXI_ARREADY,

	// Slave Interface Read Data Ports
	output reg	[C_S_AXI_DATA_WIDTH-1:0] S_AXI_RDATA,
	output reg	[2-1:0]	S_AXI_RRESP,
	output reg			S_AXI_RVALID,
	input  wire			S_AXI_RREADY,

	// Packet I/F for System Macro Trace
	input  wire			smt_sync_clk,	  		// up to 100MHz
	input  wire			smt_sync_clk_deg0_40,	// up to 100MHz
	input  wire 		trresetn,		
	output wire			clk_smt,
	output wire			xsync_smt,
	output wire	[1:0]	d_smt
	);


/***************************************************************************/
/* Parameter                                                               */
/***************************************************************************/

 parameter	RESP_OKAY 		=	2'b00;
 parameter	RESP_EXOKAY	 	=	2'b01;
 parameter	RESP_SLVERR 	=	2'b10;
 parameter	RESP_DECERR 	=	2'b11;

 parameter	IDLE_RD   	 	=	2'b00,
			RD_DATA_WAIT 	=	2'b01,
			RD_DATA_VALID 	=	2'b10;

/***************************************************************************/
/* node signals                                                            */
/***************************************************************************/
 reg				axi_awcs;
 reg		[5:2]	axi_awaddr;
 reg				axi_wen;
 reg		[31:0]	axi_wdata;

 wire			xtcs;
 wire	[5:2]	ta;
 wire	[31:0]	td;
 wire			xtwr;

 reg   			oddre1_sr1;
 reg   			oddre1_sr2;
 
 wire  			clk_smt_in;
 wire  [1:0] 	d_smt_in;
 wire  			xsync_smt_in;

/***************************************************************************/
/* State Machine for Write Access                                          */
/***************************************************************************/

	always @( posedge ACLK ) begin
		if ( ARESETN == 1'b0 ) begin
			S_AXI_AWREADY	<= 1'b0;
		end
		else begin
			if ( ~S_AXI_AWREADY && S_AXI_AWVALID && S_AXI_WVALID ) begin
				S_AXI_AWREADY	<= 1'b1;
			end
			else begin
				S_AXI_AWREADY	<= 1'b0;
			end
		end
	end

	always @( posedge ACLK ) begin
		if ( ARESETN == 1'b0 ) begin
			S_AXI_WREADY	<= 1'b0;
		end
		else begin
			if ( ~S_AXI_WREADY && S_AXI_AWVALID && S_AXI_WVALID ) begin
				S_AXI_WREADY	<= 1'b1;
			end
			else begin
				S_AXI_WREADY	<= 1'b0;
			end
		end
	end

	always @( posedge ACLK ) begin
		if ( ARESETN == 1'b0 ) begin
			S_AXI_BVALID	<= 1'b0;
			S_AXI_BRESP		<= 2'b0;
		end
		else begin
			if ( ~S_AXI_BVALID && S_AXI_AWREADY && S_AXI_AWVALID && S_AXI_WREADY && S_AXI_WVALID ) begin
				S_AXI_BVALID	<= 1'b1;
				S_AXI_BRESP		<= RESP_OKAY;
			end
			else begin
				if ( S_AXI_BVALID && S_AXI_BREADY ) begin
					S_AXI_BVALID	<= 1'b0;
				end
				else begin
					S_AXI_BVALID	<= S_AXI_BVALID;
				end
			end
		end
	end

	always @( posedge ACLK ) begin
		if ( ARESETN == 1'b0 ) begin
			axi_awcs		<= 1'b0;
			axi_awaddr[5:2]	<= 4'h0;
		end
		else begin
			if ( ~S_AXI_AWREADY && S_AXI_AWVALID && S_AXI_WVALID ) begin
				axi_awcs		<= ( S_AXI_AWADDR[31:12] == PACKET_IF_ADR_UPPER_20BIT );
				axi_awaddr[5:2]	<= S_AXI_AWADDR[5:2];
			end
			else begin
				axi_awaddr[5:2]	<= axi_awaddr[5:2];
				
				if ( S_AXI_AWREADY ) begin
					axi_awcs		<= 1'b0;
				end
				else begin
					axi_awcs		<= axi_awcs;
				end
			end
		end
	end

	always @( posedge ACLK ) begin
		if ( ARESETN == 1'b0 ) begin
			axi_wen			<= 1'b0;
			axi_wdata[31:0]	<= 32'h0;
		end
		else begin
			if ( ~S_AXI_WREADY && S_AXI_AWVALID && S_AXI_WVALID && ( S_AXI_WSTRB[3:0] == 4'b1111 )) begin
				axi_wen			<= 1'b1;
				axi_wdata[31:0]	<= S_AXI_WDATA[31:0];
			end
			else begin
				axi_wen			<= 1'b0;
				axi_wdata[31:0]	<= axi_wdata[31:0];
			end
		end
	end


/***************************************************************************/
/* State Machine for Read Access (Dummy)                                   */
/***************************************************************************/
	always @( posedge ACLK ) begin
		if ( ARESETN == 1'b0 ) begin
			S_AXI_ARREADY	<= 1'b0;
		end
		else begin
			if ( ~S_AXI_ARREADY && S_AXI_ARVALID ) begin
				S_AXI_ARREADY	<= 1'b1;
			end
			else begin
				S_AXI_ARREADY	<= 1'b0;
			end
		end
	end

	always @( posedge ACLK ) begin
		if ( ARESETN == 1'b0 ) begin
			S_AXI_RVALID	<= 1'b0;
			S_AXI_RRESP		<= 2'b0;
		end
		else begin
			if ( ~S_AXI_RVALID && S_AXI_ARREADY && S_AXI_ARVALID ) begin
				S_AXI_RVALID	<= 1'b1;
				S_AXI_RRESP		<= RESP_OKAY;
			end
			else begin
				if ( S_AXI_RVALID && S_AXI_RREADY ) begin
					S_AXI_RVALID	<= 1'b0;
				end
				else begin
					S_AXI_RVALID	<= S_AXI_RVALID;
				end
			end
		end
	end


/***************************************************************************/
/* Read Response (Dummy)                                                   */
/***************************************************************************/
	always @( posedge ACLK ) begin
		if ( ARESETN == 1'b0 ) begin
			S_AXI_RDATA	<= 32'hffff_ffff;
		end
		else begin
			if ( S_AXI_ARREADY && S_AXI_ARVALID && ~S_AXI_RVALID ) begin
				S_AXI_RDATA	<= 32'h1234_5678;
			end
			else begin
				if ( S_AXI_RVALID ) begin
					S_AXI_RDATA	<= S_AXI_RDATA;
				end
				else begin
					S_AXI_RDATA	<= 32'h0;
				end
			end
		end
	end


/***************************************************************************/
/* Packet I/F for System Macro Trace                                       */
/***************************************************************************/
	assign	xtcs		= ~axi_awcs;
	assign	ta[5:2]		= axi_awaddr[5:2];
	assign	td[31:0]	= axi_wdata[31:0];
	assign	xtwr		= ~axi_wen;

port_top port_top_inst(
	.busclk(ACLK),
	.trclk(smt_sync_clk),
	.busreset(~ARESETN),
	.trreset(~trresetn),
	.xtcs(xtcs),
	.ta(ta),
	.td(td),
	.xtwr(xtwr),
	.clk(clk_smt_in),
	.xsync(xsync_smt_in),
	.d(d_smt_in)
	);


/***************************************************************************/
// ODDRE1: Dedicated Dual Data Rate (DDR) Output Register
// UltraScale
// Xilinx HDL Libraries Guide, version 2014.1
/***************************************************************************/

//oddre1 reset signal for clk_smt
    always @(posedge smt_sync_clk_deg0_40 or negedge trresetn) begin
        if (~trresetn) begin
            oddre1_sr1 <= 1'b1;
        end
        else begin
            oddre1_sr1 <= 1'b0;
        end
	end

//clk_smt
ODDRE1 #(
	.IS_C_INVERTED(1'b0),	// Optional inversion for C
	.IS_D1_INVERTED(1'b0), // Optional inversion for D1
	.IS_D2_INVERTED(1'b0), // Optional inversion for D2
	.SRVAL(0) 				// Initializes the ODDRE1 Flip-Flops to the specified value (1fb0, 1fb1)
)
ODDRE1_inst_clk (
	.Q(clk_smt), // 1-bit output: Data output to IOB
	.C(smt_sync_clk_deg0_40), // 1-bit input: High-speed clock input
	.D1(1'b0), // 1-bit input: Parallel data input 1
	.D2(1'b1), // 1-bit input: Parallel data input 2
	.SR(oddre1_sr1) // 1-bit input: Active High Async Reset
);

//oddre1 reset signal for d_smt and xsync_smt
    always @(posedge clk_smt_in or negedge trresetn) begin
        if (~trresetn) begin
            oddre1_sr2 <= 1'b1;
        end
        else begin
            oddre1_sr2 <= 1'b0;
        end
	end
	
//d_smt
ODDRE1 #(
	.IS_C_INVERTED(1'b0),	// Optional inversion for C
	.IS_D1_INVERTED(1'b0), // Optional inversion for D1
	.IS_D2_INVERTED(1'b0), // Optional inversion for D2
	.SRVAL(0) 				// Initializes the ODDRE1 Flip-Flops to the specified value (1fb0, 1fb1)
)
ODDRE1_inst_d0 (
	.Q(d_smt[0]), // 1-bit output: Data output to IOB
	.C(clk_smt_in), // 1-bit input: High-speed clock input
	.D1(d_smt_in[0]), // 1-bit input: Parallel data input 1
	.D2(d_smt_in[0]), // 1-bit input: Parallel data input 2
	.SR(oddre1_sr2) // 1-bit input: Active High Async Reset
);

ODDRE1 #(
	.IS_C_INVERTED(1'b0),	// Optional inversion for C
	.IS_D1_INVERTED(1'b0), // Optional inversion for D1
	.IS_D2_INVERTED(1'b0), // Optional inversion for D2
	.SRVAL(0) 				// Initializes the ODDRE1 Flip-Flops to the specified value (1fb0, 1fb1)
)
ODDRE1_inst_d1 (
	.Q(d_smt[1]), // 1-bit output: Data output to IOB
	.C(clk_smt_in), // 1-bit input: High-speed clock input
	.D1(d_smt_in[1]), // 1-bit input: Parallel data input 1
	.D2(d_smt_in[1]), // 1-bit input: Parallel data input 2
	.SR(oddre1_sr2) // 1-bit input: Active High Async Reset
);


//xsync_smt

ODDRE1 #(
	.IS_C_INVERTED(1'b0),	// Optional inversion for C
	.IS_D1_INVERTED(1'b0), // Optional inversion for D1
	.IS_D2_INVERTED(1'b0), // Optional inversion for D2
	.SRVAL(0) 				// Initializes the ODDRE1 Flip-Flops to the specified value (1fb0, 1fb1)
)
ODDRE1_inst_sync (
	.Q(xsync_smt), // 1-bit output: Data output to IOB
	.C(clk_smt_in), // 1-bit input: High-speed clock input
	.D1(xsync_smt_in), // 1-bit input: Parallel data input 1
	.D2(xsync_smt_in), // 1-bit input: Parallel data input 2
	.SR(oddre1_sr2) // 1-bit input: Active High Async Reset
);


endmodule
