/************************************************************************/
/* @\F ͒i                                                    */
/*                                                                      */
/* TvF CgANZX̓AL̒os܂              */
/*        busclkɓ܂                                            */
/*                                                                      */
/*                                                                  */
/*                                                                      */
/*                                                                      */
/* ύXF                                                           */
/*  +------------------ ԍ (000 ` 999)                           */
/*  |    +------------- VKAύXAǉA폜̕                    */
/*  v    v                                                              */
/*  No     N                                               */
/* ----+------+----------+--------------------------------------------- */
/*  000 VK   10/04/26                                                 */
/*  001 ύX   16/05/12   񓯊Zbg𓯊ZbgɕύX            */
/*  002 ύX   16/10/31   AhX1bitǉ ta[4:2]ta[5:2]             */
/*                                                                      */
/************************************************************************/

module sync (
	busclk,
	busreset,
	xtwr,
	xtcs,
	ta,
	td,
	datain,
	wr,
	full
	);

/***************************************************************************/
/* DECLARE THE PORT DIRECTION                                              */
/***************************************************************************/

 input			busclk;
 input			busreset;
 input			xtwr;
 input			xtcs;
 input	[5:2]	ta;
 input	[31:0]	td;
 output	[36:0]	datain;
 output			wr;
 input			full;


/***************************************************************************/
/* DECLARE THE NODE(WIRE/REG)                                              */
/***************************************************************************/
 reg	[5:2]	ta_1l,ta_2l,ta_3l;
 reg	[31:0]	td_1l,td_2l,td_3l;
 reg			wr_1l,wr_2l;
 reg			cs_1l,cs_2l;
 wire			wr_p_2l;
 reg			wr_p_3l;

 reg	[36:0]	sync_data;
 reg			sync_wr;

 wire			header_det_2l;

 reg 			wr_msk;
 reg			lost;
 reg	[1:0]	write_msk_st;


/***************************************************************************/
/* patameter                                                               */
/***************************************************************************/
parameter
 WAIT_FULL     = 2'b00,
 WAIT_FULL_NEG = 2'b01,
 WAIT_HEADER   = 2'b10;

/***************************************************************************/
/* output                                                                  */
/***************************************************************************/
	assign	datain	= sync_data;
	assign	wr		= sync_wr;

/***************************************************************************/
/* pipeline 1'st                                                           */
/***************************************************************************/
	always @(posedge busclk) begin
		if (busreset) begin
			wr_1l	<= 1'b0;
			cs_1l	<= 1'b0;
			ta_1l	<= 4'b0;
			td_1l	<= 32'hffff_ffff;
		end
		else begin
			wr_1l	<= ~xtwr;
			cs_1l	<= ~xtcs;
			ta_1l	<= ta;
			td_1l	<= td;
		end
	end

/***************************************************************************/
/* pipeline 2'nd                                                           */
/***************************************************************************/
	always @(posedge busclk) begin
		if (busreset) begin
			wr_2l	<= 1'b0;
			cs_2l	<= 1'b0;
			ta_2l	<= 4'b0;
			td_2l	<= 32'hffff_ffff;
		end
		else begin
			wr_2l	<= wr_1l;
			cs_2l	<= cs_1l;
			ta_2l	<= ta_1l;
			td_2l	<= td_1l;
		end
	end

	assign	wr_p_2l	= cs_2l & ~wr_1l & wr_2l;

/***************************************************************************/
/* pipeline 3'rd                                                           */
/***************************************************************************/
	always @(posedge busclk) begin
		if (busreset) begin
			ta_3l	<= 4'b0;
			td_3l	<= 32'hffff_ffff;
		end
		else begin
			ta_3l	<= ta_2l;
			td_3l	<= td_2l;
		end
	end

	always @(posedge busclk) begin
		if (busreset) begin
			wr_p_3l	<= 1'b0;
		end
		else begin
			wr_p_3l	<= wr_p_2l;
		end
	end

/***************************************************************************/
/* pipeline 4'th                                                           */
/***************************************************************************/
	always @(posedge busclk) begin
		if (busreset) begin
			sync_wr	  <= 1'b0;
			sync_data <= 37'h1_ffff_ffff_f;
		end
		else begin
			sync_wr			<= ( wr_msk | full ) ? 1'b0 : wr_p_3l;
			sync_data[36]	<= lost;
			sync_data[35:4]	<= td_3l[31:0];
			sync_data[3:1]	<= ta_3l[4:2];
			sync_data[0]	<= ta_3l[5];
		end
	end

/***************************************************************************/
/* header detector                                                         */
/***************************************************************************/
	assign	header_det_2l	= (ta_2l[2] == 1'b0) & wr_p_2l;

/***************************************************************************/
/* fifo write mask                                                         */
/***************************************************************************/
	always @(posedge busclk) begin
		if (busreset) begin
			wr_msk			<= 1'b0;
			lost			<= 1'b0;
			write_msk_st	<= WAIT_FULL;
		end
		else begin
			case (write_msk_st)
			WAIT_FULL : begin
				if (full == 1'b1) begin
					wr_msk			<= 1'b1;
					lost			<= 1'b1;
					write_msk_st	<= WAIT_FULL_NEG;
				end
				else begin
					wr_msk			<= 1'b0;
					lost			<= 1'b0;
					write_msk_st	<= WAIT_FULL;
				end
			end
			WAIT_FULL_NEG : begin
				wr_msk				<= 1'b1;
				lost				<= 1'b1;
				
				if (full == 1'b0) begin
					write_msk_st	<= WAIT_HEADER;
				end
				else begin
					write_msk_st	<= WAIT_FULL_NEG;
				end
			end
			WAIT_HEADER : begin
				if (header_det_2l == 1'b1) begin
					wr_msk			<= 1'b0;
					lost			<= 1'b1;
					write_msk_st	<= WAIT_FULL;
				end
				else begin
					wr_msk			<= 1'b1;
					lost			<= 1'b1;
					write_msk_st	<= WAIT_HEADER;
				end
			end
			default : begin
				wr_msk			<= 1'b0;
				lost			<= 1'b0;
				write_msk_st	<= WAIT_FULL;
			end
			endcase
		end
	end


endmodule
